==> ideaIC-143.747.6-sources.jar: com/intellij/xml/XmlAttributeDescriptor.java <==
/*
 * Copyright 2000-2009 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.intellij.xml;

import com.intellij.psi.meta.PsiMetaData;
import com.intellij.psi.xml.XmlElement;
import com.intellij.util.ArrayFactory;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * @author Mike
 */
public interface XmlAttributeDescriptor extends PsiMetaData {
  XmlAttributeDescriptor[] EMPTY = new XmlAttributeDescriptor[0];
  ArrayFactory<XmlAttributeDescriptor> ARRAY_FACTORY = new ArrayFactory<XmlAttributeDescriptor>() {
    @NotNull
    @Override
    public XmlAttributeDescriptor[] create(int count) {
      return new XmlAttributeDescriptor[count];
    }
  };

  boolean isRequired();
  boolean isFixed();
  boolean hasIdType();
  boolean hasIdRefType();

  @Nullable
  String getDefaultValue();

  //todo: refactor to hierarchy of value descriptor?
  boolean isEnumerated();
  @Nullable
  String[] getEnumeratedValues();

  @Nullable
  String validateValue(XmlElement context, String value);
}

==> ideaIC-2019.2.1-sources.jar: com/intellij/xml/XmlAttributeDescriptor.java <==
/*
 * Copyright 2000-2009 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.intellij.xml;

import com.intellij.psi.PsiElement;
import com.intellij.psi.meta.PsiMetaData;
import com.intellij.psi.xml.XmlElement;
import com.intellij.util.ArrayFactory;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Collection;
import java.util.Collections;

/**
 * @author Mike
 */
public interface XmlAttributeDescriptor extends PsiMetaData {
  XmlAttributeDescriptor[] EMPTY = new XmlAttributeDescriptor[0];
  ArrayFactory<XmlAttributeDescriptor> ARRAY_FACTORY = count -> count == 0 ? EMPTY : new XmlAttributeDescriptor[count];

  boolean isRequired();
  boolean isFixed();
  boolean hasIdType();
  boolean hasIdRefType();

  @Nullable
  String getDefaultValue();

  //todo: refactor to hierarchy of value descriptor?
  boolean isEnumerated();
  @Nullable
  String[] getEnumeratedValues();

  @Nullable
  String validateValue(XmlElement context, String value);

  @NotNull
  default Collection<PsiElement> getDeclarations() {
    PsiElement declaration = getDeclaration();
    return declaration != null ? Collections.singleton(declaration)
                               : Collections.emptyList();
  }
}
